# ------------------------------------------------------------------------------
# ----------------------------------------------------------------
#  rftClient.py: Reliable File Transfer client library routines
#     classes:
#         rftClient
#     functions:
#         setDefaults()
# ----------------------------------------------------------------
import sys, socket, getopt, ConfigParser, traceback
import pyGlobus
from pyGlobus import GSISOAP
from pyGlobus.GSISOAP import ioc, SOAPProxy

requests = ('completeTransferRequest', 'submitTransferRequest' ,
            'commitTransferRequest', 'listTransferRequest' ,
            'checkStatusRequest' , 'changePriorityRequest',
            'clearRecordRequest' , 'userStopTransferRequest' ,
            'printDS' , 'getDump')
# ----------------------------------------------------------------
#  rftClient:
#
# ----------------------------------------------------------------
class rftClient:
     def __init__(self, protocol, hostname, port, namespace):
          self._server = SOAPProxy(protocol + hostname + ':' + str(port),
                                   namespace)
          self._server.channel_mode = ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_SSL_WRAP
          self._server.delegation_mode = \
                                       ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_NONE
          return
     def completeTransferRequest(self, handle, proto, src, dst, priority):
          h = self._server.completeTransferRequest(handle, proto,
                                                   src, dst, priority)
          return h
     def submitTransferRequest(self, handle, proto, src, dst, priority):
          h = self._server.submitTransferRequest(handle, proto,
                                                 src, dst, priority)
          return h
     def commitTransferRequest(self, handle):
          h = self._server.commitTransferRequest(handle)
          return h
     def listTransferRequest(self, handle):
          h = self._server.listTransferRequest(handle)
          return h
     def checkStatusRequest(self, handle):
          s = self._server.checkStatusRequest(handle)
          return s
     def changePriorityRequest(self, handle, priority):
          h = self._server.changePriorityRequest(handle, priority)
          return h
     def clearRecordRequest(self, handle):
          h = self._server.clearRecordRequest(handle)
          return h
     def userStopTransferRequest(self, handle):
          h = self._server.userStopTransferRequest(handle)
          return h
     def printDS(self):
          tmp = self._server.printDS()
          return tmp
     def getDump(self):
          return self._server.getDump()

# ----------------------------------------------------------------
#  setDefaults: read default values for [Client] from
#      default.cfg
# ----------------------------------------------------------------
def setDefaults():
     cp = ConfigParser.ConfigParser()
     cp.readfp(open('defaults.cfg'))
     cp_section = 'Client'
     port            = cp.getint(cp_section, 'port')
     namespace       = cp.get(cp_section, 'namespace')
     proto           = (cp.get(cp_section, 'trans_protocol'))
     soap_protocol   = cp.get(cp_section, 'soap_protocol')
     debug           = cp.getint(cp_section, 'debug_level')
     del cp
     del cp_section
     return (port, namespace, proto, soap_protocol, debug)
