# ----------------------------------------------------------------
# ------------------------------------------------------------------------------
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
#
# ----------------------------------------------------------------
import DataManager
#from DataManager import DataB, DataG
import unittest
import ConfigParser
import sys, random, time, os
from exceptions import Exception

g_dataBaseInt = None

class DataBaseInterfaceTestCase(unittest.TestCase):
    def setUp(self):
        try:
            print "DataBaseInterface: %s" %(self._dBI)
        except AttributeError, ex:
            self._cp = ConfigParser.ConfigParser()
            self._cp.readfp(open('test.cfg'))
            self._cp_section = 'DataBaseInterface'
            if g_dataBaseInt == 'DataG':
                name  = self._cp.get(cp_section, 'file')
                table = self._cp.get(cp_section, 'table')
                dir   = self._cp.get(cp_section, 'dir')
                dir += 'DataG'
                self._dBI = DataManager.DataG(name, dir, table)
            elif g_dataBaseInt == 'DataB':
                file_name  = self._cp.get(cp_section, 'file')
                dir   = self._cp.get(cp_section, 'dir')
                dir += 'DataB'
                self._dBI = DataManager.DataB(dir, file_name)
            else:
                raise RuntimeError('Do not recognize DBI',
                                   str(g_dataBaseInt))

            self._proto    = self._cp.get(cp_section, 'protocol')
            self._url_src  = self._cp.get(cp_section, 'src')
            self._url_dst  = self._cp.get(cp_section, 'dst')
            self._cred     = self._cp.get(cp_section, 'cred')
            self._marker   = ''
            entries  = self._cp.getint(cp_section, 'entries')
            random.seed()
            for i in range(entries):
                self._handle   = str(random.random())
                self._priority = i
                try:
                    self._dBI.insert(self._handle, self._proto, self._priority,
                                     self._url_send, self._url_recv, self._cred)
                except Exception, ex:
                    self.failUnless(1, ('failed to insert random handle'
                                        + str(ex)))
        return
    def tearDown(self):
        return
    #----------------------------------------------------------
    # to_string(self)
    # isEmpty(self)
    # insert(self, handle, proto, priority, url_src, url_dest, credPath='')
    # clear_entry(self, handle)
    # get_credPath(self, handle)
    # set_credPath(self, handle)
    # get_marker(self, handle)
    # set_marker(self, handle, marker)
    # get_priority(self, handle)
    # set_priority(self, handle, priority)
    # get_status(self, handle)
    # set_status(self, handle, status)
    # get_tuple_of_handles_of_status(self,status)
    # get_transfer_info(self, handle)
    # get_record(self, handle)
    #----------------------------------------------------------
    def test_double_insert(self):
        proto    = self._cp.get(cp_section, 'protocol')
        url_src  = self._cp.get(cp_section, 'src')
        url_dst  = self._cp.get(cp_section, 'dst')
        cred     = self._cp.get(cp_section, 'cred')
        handle   = str(random.random())
        priority = 0
        try:
            self._dBI.insert(handle, proto, priority,
                             url_send, url_recv, cred)
        except Exception, ex:
            self.failUnless(1, ('failed to insert random handle: ' + str(ex)))
            return
        else:
            try:
                self._dBI.insert(handle, proto, (not priority),
                                 url_send, url_recv, cred)
            except ValueError, ex: pass
            except Exception, ex:
                self.failUnless(1, ('wrong exception: ' + str(ex)))
            else:
                self.failUnless(1, 'inserted previosly inserted handle')
                return
        return
    def test_clear_entry(self):
        proto    = self._cp.get(cp_section, 'protocol')
        url_src  = self._cp.get(cp_section, 'src')
        url_dst  = self._cp.get(cp_section, 'dst')
        cred     = self._cp.get(cp_section, 'cred')
        handle   = str(random.random())
        priority = 0
        try:
            self._dBI.insert(handle, proto, priority,
                             url_send, url_recv, cred)
        except Exception, ex:
            self.failUnless(1, ('clear_entry failed at insert: ' + str(ex)))
            return
        else:
            self._dBI.clear_entry(handle)
            try:
                self.get_marker(handle)
            except ValueError, ex:
                pass
            else:
                self.failUnless(1, 'clear_entry failed to clear')
        return
    def test_accessors(self):
        # Attempt to use all accessors legally
        proto    = self._cp.get(cp_section, 'protocol')
        url_src  = self._cp.get(cp_section, 'src')
        url_dst  = self._cp.get(cp_section, 'dst')
        cred     = self._cp.get(cp_section, 'cred')
        handle   = str(random.random())
        priority = 0
        try:
            self._dBI.insert(handle, proto, priority,
                             url_send, url_recv, cred)
        except Exception, ex:
            self.failUnless(1, ('failed to insert random handle: ' + str(ex)))
            return
        try:
            e = self._dBI.isEmpty()
            c = self._dBI.get_credPath(handle)
            c += 'hello'
            self._dBI.set_credPath(handle, c)
            c2 = self._dBI.get_credPath(handle)
            self.failUnless((c == c2),
                            ('get_credPath returned incorrect %s expected %s'
                             %(c2, c)))
            m = self._dBI.get_marker(handle)
            m += 'hello'
            self._dBI.set_marker(handle, (m + 'hello'))
            m2 = self._dBI.get_marker(handle)
            self.failUnless((m == m2),
                            ('get_marker returned incorrect %s expected %s'
                             %(c2, c)))
            p = self._dBI.get_priority(handle)
            p = not p
            self._dBI.set_priority(handle, p)
            p2 = self._dBI.get_priority(handle)
            self.failUnless((p == p2),
                            ('get_priority returned incorrect %d expected %d'
                             %(p, p2)))
            
            s = self._dBI.get_status(handle)
            s += 1
            self._dBI.set_status(handle, s)
            s2 = self._dBI.get_status(handle)
            self.failUnless((s == s2),
                            ('get_status returned incorrect %d expected %d'
                             %(s, s2)))
            t  = self._dBI.get_tuple_of_handles_of_status(s2)
            ti = self._dBI.get_transfer_info(handle)
            r  = self._dBI.get_record(handle)
        except Exception, ex:
            self.failUnless(1, ('exception: ' + str(ex)))
        return
    def test_bad_handle(self):
        handle = str(random.random())
        
        try: c = self._dBI.get_credPath(handle)
        except ValueError, ex: pass
        else: self.failUnless(1, 'get_credPath bad handle')
        
        try: c = self._dBI.get_marker(handle)
        except ValueError, ex: pass
        else: self.failUnless(1, 'get_marker bad handle')
        try: self._dBI.set_marker(handle, 'bad')
        except ValueError, ex: pass
        else: self.failUnless(1, 'set_marker bad handle')

        try: c = self._dBI.get_priority(handle)
        except ValueError, ex: pass
        else: self.failUnless(1, 'get_priority bad handle')
        try: self._dBI.set_priority(handle, 0)
        except ValueError, ex: pass
        else: self.failUnless(1, 'set_priority bad handle')

        try: c = self._dBI.get_status(handle)
        except ValueError, ex: pass
        else: self.failUnless(1, 'get_status bad handle')
        try: self._dBI.set_status(handle, 3)
        except ValueError, ex: pass
        else: self.failUnless(1, 'set_status bad handle')

        try: c = self._dBI.get_transfer_info(handle)
        except ValueError, ex: pass
        else: self.failUnless(1, 'get_transfer_info bad handle')

        try: c = self._dBI.get_record(handle)
        except ValueError, ex: pass
        else: self.failUnless(1, 'get_record bad handle')
        return
    def test_to_string(self):
        try:
            r = self._dBI.to_string(self._handle)
        except Exception, ex:
            self.failUnless(1, ("no to_string() for " + self._handle
                                + ': ' + str(ex)))
        else:
            tok = '#'
            for i in [self._handle, (self._proto + self._url_src),
                      (self._proto + self._url_dst), self._status,
                      self._priority, self._marker]:
                end = r.find(tok)
                rh = r[:end]
                r = r[(end + 1):]
                self.failUnless( rh == i, 'parse to_string() incorrectly')
        return
    def test_get_transfer_info(self):
        try:
            t = self._dBI.get_transfer_info(self._handle)
        except Exception, ex:
            self.failUnless(1, ("no get_transfer_info() for " + self._handle
                                + ': ' + str(ex)))
        else:
            self.failUnless((type(t) == type(())),
                            ("expected tuple " + self._handle))
            self.failUnless((len(t) == 5), ("expected 5 tuple " + self._handle))
            l = [self._proto, self._url_src, self._url_dst, self._marker,
                 self._cred]
            for i in range(len(l)):
                self.failUnless(
                    (l[i] == t[i]),
                    ("returned %s\n\tfield %d- incorrect(%s) expected(%s)"
                     %(str(t), i, t[i], l[i])))
    def test_get_record(self):
        try:
            t = self._dBI.get_record(self._handle)
        except Exception, ex:
            self.failUnless(1, ("no get_record() for " + self._handle
                                + str(ex)))
        else:
            self.failUnless((type(t) == type(())),
                            ("expected tuple " + self._handle))
            self.failUnless((len(t) == 6),
                            ("expected 6 tuple " + self._handle))
            l = [self._proto, self._url_src, self._url_dst, self._marker,
                 0, self._priority]
            for i in range(len(l)):
                self.failUnless(
                    (l[i] == t[i]),
                    ("returned %s\n\tfield %d- incorrect(%s) expected(%s)"
                     %(str(t), i, t[i], l[i])))
#----------------------------------------------------------
#  main
#
#----------------------------------------------------------
def makeTestSuite():
    set_globals()
    testSuite = unittest.makeSuite(DataBaseInterfaceTestCase,'test')
    return testSuite
def set_globals():
    global g_dataBaseInt, cp_section
    cp = ConfigParser.ConfigParser()
    cp.readfp(open('test.cfg'))
    cp_section = 'DataBaseInterface'
    #g_dataBaseInt = eval(cp.get(cp_section, 'dbinterface'))
    g_dataBaseInt = cp.get(cp_section, 'dbinterface')
    del cp
    print "DatBaseInterface: ", g_dataBaseInt
    return
def main():
    unittest.main(defaultTest="makeTestSuite")
    return

if __name__ == "__main__" : main()
